/* Definitions file for GNU Emacs running on QNX 4.2x
   Copyright (C) 1985, 1986, 1994 Free Software Foundation, Inc.

   Author: Andrew Thomas, Cogent Real-Time Systems Inc. 1998

This file is part of GNU Emacs.

GNU Emacs is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

GNU Emacs is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with GNU Emacs; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */


/*
 *	Define symbols to identify the version of Unix this is.
 *	Define all the symbols that apply correctly.
 */

#ifndef __QNX__
#define __QNX__
#endif /* __QNX__ */

/* Some includes really should be here... */

#ifdef emacs
#ifndef _STDLIB_H_INCLUDED
#include	<stdlib.h>
#undef min
#undef max
#endif

#ifndef _MATH_H_INCLUDED
#include	<math.h>
#endif

#ifndef _STRING_H_INCLUDED
#include	<string.h>
#endif
#endif

/* SYSTEM_TYPE should indicate the kind of system you are using.
 It sets the Lisp variable system-type.  */

#define SYSTEM_TYPE "QNX 4.2x"

/* nomultiplejobs should be defined if your system shell
 does not have "job control" (the ability to stop a program,
 run some other program, then continue the first one).  */

#define NOMULTIPLEJOBS

/* Emacs can read input using SIGIO and buffering characters itself,
   or using CBREAK mode and making C-g cause SIGINT.
   The choice is controlled by the variable interrupt_input.

   Define INTERRUPT_INPUT to make interrupt_input = 1 the default (use SIGIO)

   Emacs uses the presence or absence of the SIGIO macro to indicate
   whether or not signal-driven I/O is possible.  It uses
   INTERRUPT_INPUT to decide whether to use it by default.

   SIGIO can be used only on systems that implement it (4.2 and 4.3).
   CBREAK mode has two disadvantages
     1) At least in 4.2, it is impossible to handle the Meta key properly.
        I hear that in system V this problem does not exist.
     2) Control-G causes output to be discarded.
        I do not know whether this can be fixed in system V.

   Another method of doing input is planned but not implemented.
   It would have Emacs fork off a separate process
   to read the input and send it to the true Emacs process
   through a pipe. */

/* #define INTERRUPT_INPUT */

/* First pty name is /dev/ptyp0.  */

#define FIRST_PTY_LETTER 'p'

/*
 *	Define HAVE_PTYS if the system supports pty devices.
 */

#define HAVE_PTYS

/* Define this macro if system defines a type 'union wait'.  */

#define HAVE_UNION_WAIT

/* Define this macro if system has a wait header.  */

#define HAVE_WAIT_HEADER
#define WAIT_USE_INT
#define WCOREDUMP(x)	0

#define wait3(statloc,options,pid)	waitpid(-1,statloc,options)

/* Define HAVE_SOCKETS if system supports 4.2-compatible sockets.  */

#define HAVE_SOCKETS

/* If we have utime.h */

#define HAVE_UTIMES

/*
 *	Define NONSYSTEM_DIR_LIBRARY to make Emacs emulate
 *      The 4.2 opendir, etc., library functions.
 */

/* #define NONSYSTEM_DIR_LIBRARY */

/* Define this symbol if your system has the functions bcopy, etc. */

#define BSTRING

/* subprocesses should be defined if you want to
   have code for asynchronous subprocesses
   (as used in M-x compile and M-x shell).
   This is generally OS dependent, and not supported
   under most USG systems. */

#define subprocesses

/* If your system uses COFF (Common Object File Format) then define the
   preprocessor symbol "COFF". */

/* #define COFF */

/* define MAIL_USE_FLOCK if the mailer uses flock
   to interlock access to /usr/spool/mail/$USER.
   The alternative is that a lock file named
   /usr/spool/mail/$USER.lock.  */

/* #define MAIL_USE_FLOCK */

/* Define CLASH_DETECTION if you want lock files to be written
   so that Emacs can tell instantly when you try to modify
   a file that someone else has modified in his Emacs.  */

#define CLASH_DETECTION

/* We use the Berkeley (and usg5.2.2) interface to nlist.  */

/* #define NLIST_STRUCT */

/* The file containing the kernels symbol table is called /vmunix.  */

/* #define KERNEL_FILE "/vmunix" */

/* The symbol in the kernel where the load average is found
   is named _avenrun.  */

/* #define LDAV_SYMBOL "_avenrun" */

/* Send signals to subprocesses by "typing" special chars at them.  */

/* #define SIGNALS_VIA_CHARACTERS */

/* We do have vfork.  */

#define HAVE_VFORK

/* Process groups work in the traditional BSD manner.  Not in QNX. */

/* #define BSD_PGRPS */

/* We have alloca */

#define HAVE_ALLOCA

/* Our getpgrp has no arg */

#define GETPGRP_NO_ARG

/* Watcom C forces "interrupt" to "__interrupt" */

#define interrupt xinterrupt

/* We have TERMIOS */

#define HAVE_TERMIOS

/* We use dirent, not direct for directory entries */

#define SYSV_SYSTEM_DIR

/* We have gethostname() as part of the TCP/IP dev kit.  If we do not
   use it, we get the node number instead of a host name. */

/* #define HAVE_GETHOSTNAME */

/* The linker should just use the regular crt0.o, not the GNU one */

#define ORDINARY_LINK

/* QNX can UNEXEC now.  Excellent. */

/* #define CANNOT_DUMP */

/* Oops.  They use _longjmp and longjmp interchangeably */

#define _longjmp longjmp

/* We do not know the start of the text segment at runtime.  We get it
   from the executable file. */

#undef NEED_STARTS
#define HAVE_TEXT_START

/* Use our own malloc?  Naw. */

#undef SYSTEM_MALLOC

/* Extra C flags for this system.  Quiet output, pack structs to 2 bytes.
 * Now for the bad news.  We really should be compiling with stack argument
 * passing (-4s), since Emacs is full of "kluge" vararg functions.  We cannot
 * do this because there is no unix3s.lib for Watcom 10.6.  The only option is
 * to tell the compiler that those functions damage all registers and hope
 * there are no other side effects.
 */

#define CFLAGS				NOFLAGS
#define C_SWITCH_SYSTEM		-zp2 -g1 -Onetx -w3 "-Wc,-v"

/* Extra Link flags for this system.  Quiet output */

#define LD_SWITCH_SYSTEM	-zp2 -Q -g1 -Onetx -N 512K
#define LIBS_DEBUG
#define LIB_MATH

/* Functions Emacs does not prototype that have fixed arg lists but are used
   as vararg functions...  We have to walk a little bit of hell to get these
   to do the right thing:
   1) Do not let Watcom C pass any args as registers.  That is what the
      "parm []" section should do for use.
   2) Do not let Watcom C assume that any registers are safe on entry to the
      function.  This may not be necessary now.
   3) Prototype the functions to be vararg functions.  This is done in
      qnxproto.h.  This might make a lot of this register passing limitation
	  unnecessary, but why take chances?
   4) Add an ellipsis to the definition of the function.  This is done in
      the function code in various files.
 */

#ifdef emacs
#pragma aux format1 modify [eax ebx ecx edx esi edi fs gs] parm [];
#pragma aux error modify [eax ebx ecx edx esi edi fs gs] parm [];
#pragma aux sys_open modify [eax ebx ecx edx esi edi fs gs] parm [];
#pragma aux fatal modify [eax ebx ecx edx esi edi fs gs] parm [];
#pragma aux tparam modify [eax ebx ecx edx esi edi fs gs] parm [];
#pragma aux message modify [eax ebx ecx edx esi edi fs gs] parm [];
#endif

/* We have the string.h header */

#define HAVE_STRING_H

/* mktemp is really tmpnam here */

#define mktemp	tmpnam

/* Watcom C uses register arg passing, so... */

#define NO_ARG_ARRAY

/* We do not have NSIG but we do have _SIGMAX */

#define NSIG _SIGMAX

/* Watcom C naturally uses IEEE floating point */

#define IEEE_FLOATING_POINT 1

/* We want our own unexec function... */

#define UNEXEC unexqnx4.o

/* Maybe we can use the mouse in Photon and QNX Windows */

#define HAVE_MOUSE

/* Fake out the makefile to add qnx4.o */

#define MSDOS_OBJ qnx4.o

/* They use croak(char* msg).  We do not have it */

#ifdef emacs
#define croak(fname)	(printf ("Croaked in %s\r\n", fname), \
						 reset_sys_modes (), exit (1))
#endif

#if defined(emacs) && !defined(MOVEMAIL)
#include "s/qnxproto.h"
/* QNX defines SIGIO, but doesn't really mean it. */
#undef SIGIO
#endif
