/*
 * Unexec for QNX4.2x systems
 *
 * Author: Andrew Thomas, Cogent Real-Time Systems Inc., April 1998
 */

#ifndef CANNOT_DUMP

#include	<stdio.h>
#include	<stdlib.h>
#include	<limits.h>
#include	<sys/lmf.h>
#include	<sys/types.h>
#include	<sys/stat.h>

static int copy_file_data (FILE* infile, FILE* outfile, int len)
{
	char		buf[1024];
	int			nread, maxread;

	while (len > 0)
	{
		maxread = (len > 1024 ? 1024 : len);
		nread = fread (buf, 1, maxread, infile);
		if (nread > 0)
		{
			len -= nread;
			fwrite (buf, nread, 1, outfile);
		}
		else
			break;
	}
	return (len);
}

static int copy_record (struct _lmf_header* head, FILE* infile, FILE* outfile,
						int keep_position)
{
	long		pos;

	pos = ftell (infile);
	fwrite (head, sizeof(*head), 1, outfile);
	copy_file_data (infile, outfile, head->data_nbytes);
	if (keep_position)
		fseek (infile, pos, SEEK_SET);
	return (0);
}

unexec (new_name, a_name, data_start, bss_start, entry_address)
     char *new_name, *a_name;
     unsigned data_start, bss_start, entry_address;
{
	int						retval = -1, fd;
	FILE					*infile, *outfile;
	struct _lmf_definition	def;
	struct _lmf_header		head;
	struct _lmf_data		data;
	unsigned long			tsize, dsize, codeaddr, dataaddr, dsizepos;
	unsigned long			putdata = 0, curpos, outpos;
	struct stat				statbuf;

	if (infile = fopen (a_name, "r"))
	{
		fstat (fileno(infile), &statbuf);
		if (outfile = fopen (new_name, "w"))
		{
			for (;;)
			{
				fread (&head, sizeof(head), 1, infile);
				// printf ("Record type: %d\n", head.rec_type);
				
				if (head.rec_type == _LMF_DEFINITION_REC)
				{
					copy_record (&head, infile, outfile, 1);
					fread (&def, sizeof(def), 1, infile);
					fread (&tsize, sizeof(tsize), 1, infile);
					dsizepos = ftell (infile);
					fread (&dsize, sizeof(dsize), 1, infile);
					
					if (sizeof(def) + 2*sizeof(tsize) != head.data_nbytes)
					{
						// printf ("Head record wrong size\n");
						fseek (infile, head.data_nbytes - sizeof(def),
							   SEEK_CUR);
					}

					tsize &= ~0xf0000000;
					codeaddr = def.unmapped_size + def.flat_offset;
					dataaddr = (codeaddr + tsize + 0xfff) & ~0xfff;
					curpos = (unsigned long)sbrk(0);
					dsize = curpos - dataaddr;

					/* Rewrite the data size */
					outpos = ftell (outfile);
					fseek (outfile, dsizepos, SEEK_SET);
					fwrite (&dsize, sizeof(dsize), 1, outfile);
					fseek (outfile, outpos, SEEK_SET);

					//printf ("Code: %d bytes at %d, Data: %d bytes at %d\n",
					//		tsize, codeaddr, dsize, dataaddr);
				}
				else if (head.rec_type == _LMF_DATA_REC)
				{
					fread (&data, sizeof(data), 1, infile);
					fwrite (&head, sizeof(head), 1, outfile);
					fwrite (&data, sizeof(data), 1, outfile);
					//printf ("Code for segment %d at offset %d\n",
					//		data.segment_index, data.offset);

					if (data.segment_index == 1)  /* data segment */
					{
						fwrite ((char*)dataaddr + data.offset,
								head.data_nbytes - sizeof(data), 1, outfile);
						putdata += head.data_nbytes - sizeof(data);
					}
					else
					{
						fwrite ((char*)codeaddr + data.offset,
								head.data_nbytes - sizeof(data), 1, outfile);
					}
					fseek (infile, head.data_nbytes - sizeof(data), SEEK_CUR);
				}
				else if (head.rec_type == _LMF_ENDDATA_REC)
				{
					/* There was local data to put */
					while (putdata < dsize)
					{
						struct _lmf_header		head2;
						struct _lmf_data		data2;
						long					nbytes;

						memset (&head2, 0, sizeof(head2));
						head2.rec_type = _LMF_DATA_REC;
						nbytes = sizeof(data2) + dsize - putdata;
						if (nbytes > 16000)
							nbytes = 16000;
						head2.data_nbytes = nbytes;
						data2.segment_index = 1;
						data2.offset = putdata;
						fwrite (&head2, sizeof(head2), 1, outfile);
						fwrite (&data2, sizeof(data2), 1, outfile);
						fwrite ((char*)dataaddr + data2.offset,
								head2.data_nbytes - sizeof(data2), 1, outfile);
						putdata += head2.data_nbytes - sizeof(data2);
					}
					/* Put the end of data in place */
					copy_record (&head, infile, outfile, 0);
				}
				else if (head.rec_type == _LMF_EOF_REC)
				{
					copy_record (&head, infile, outfile, 0);
					if (ftell (infile) < statbuf.st_size)
					{
						copy_file_data (infile, outfile,
										statbuf.st_size - ftell(infile));
					}
					break;
				}
				else
				{
					copy_record (&head, infile, outfile, 0);
				}
			}
			fclose (outfile);
			chmod (new_name, 0775);
			retval = 0;
		}
		fclose (infile);
	}
	return (retval);
}

#ifdef TESTING
int main (int argc, char** argv)
{
	char					myname[PATH_MAX];

	if (argc > 1)
	{
		_fullpath (myname, argv[0], sizeof(myname));
		unexec (argv[1], myname, 0, 0, 0);
	}
	return (0);
}
#endif

#endif /* CANNOT_DUMP */
